#!/usr/bin/env python3
#-*- coding: utf-8 -*-
import sys
sys.path.append('/Volumes/Sandisk/Projects/bravaorm')

from logging import debug

import os
import re
import json



import bravaorm

from model import *
from model import Blackwords

import time
import json


def main(arg):


    start = time.time()

    conn = bravaorm.Connection(db_user="root", db_password="root", db_host="localhost", db_port=3306, db_database="gofans", log_level="debug")


    test_blackword = Blackwords()
    test_blackword.bla_word = "teste py12"
    conn.add(test_blackword)

    test_status_pagamento = StatusPagamento()
    test_status_pagamento.status_pagamento = "Statuspy12"
    conn.add(test_status_pagamento)

    conn.save()

    print("\n\nFim conversa")
    conn.close()

if __name__ == "__main__":
    main(sys.argv)


