import sys
sys.path.append('/Volumes/Sandisk/Projects/bravaorm')
import bravaorm

def main():
    conn = bravaorm.Connection(db_user="root", db_password="root", db_host="localhost", db_port=3306, db_database="gofans", log_level="debug")
    adms = conn.administradores.include("modulos").all
    print("OUTPUT FORMAT WITH REPR:")
    print(adms)
    conn.close()

if __name__ == "__main__":
    main()
